@@ -79,14 +79,17 @@ def node_factory(edge_model, children_null=True, base_model=models.Model):
 
         def add_child(self, child, **kwargs):
             kwargs.update({"parent": self, "child": child})
-            disable_check = kwargs.pop("disable_circular_check", False)
+
+            disable_circular_check = kwargs.pop("disable_circular_check", False)
+            allow_duplicate_edges = kwargs.pop("allow_duplicate_edges", True)
+
             cls = self.children.through(**kwargs)
-            return cls.save(disable_circular_check=disable_check)
+            return cls.save(disable_circular_check=disable_circular_check, allow_duplicate_edges=allow_duplicate_edges)
 
         def remove_child(self, child, delete_node=False):
             """Removes the edge connecting this node to child, and optionally deletes the child node as well"""
             if child in self.children.all():
-                self.children.through.objects.get(parent=self, child=child).delete()
+                self.children.through.objects.filter(parent=self, child=child).delete()
                 if delete_node:
                     # Note: Per django docs:
                     # https://docs.djangoproject.com/en/dev/ref/models/instances/#deleting-objects
@@ -100,7 +103,7 @@ def node_factory(edge_model, children_null=True, base_model=models.Model):
         def remove_parent(self, parent, delete_node=False):
             """Removes the edge connecting this node to parent, and optionally deletes the parent node as well"""
             if parent in self.parents.all():
-                parent.children.through.objects.get(parent=parent, child=self).delete()
+                parent.children.through.objects.filter(parent=parent, child=self).delete()
                 if delete_node:
                     # Note: Per django docs:
                     # https://docs.djangoproject.com/en/dev/ref/models/instances/#deleting-objects
@@ -379,6 +382,11 @@ def node_factory(edge_model, children_null=True, base_model=models.Model):
             if child in parent.self_and_ancestors():
                 raise ValidationError("The object is an ancestor.")
 
+        @staticmethod
+        def duplicate_edge_checker(parent, child):
+            if child in parent.self_and_descendants():
+                raise ValidationError("The edge is a duplicate.")
+
     return Node
 
 
@@ -459,6 +467,10 @@ def edge_factory(
         def save(self, *args, **kwargs):
             if not kwargs.pop("disable_circular_check", False):
                 self.parent.__class__.circular_checker(self.parent, self.child)
+
+            if not kwargs.pop("allow_duplicate_edges", True):
+                self.parent.__class__.duplicate_edge_checker(self.parent, self.child)
+
             super().save(*args, **kwargs)
 
     return Edge
